package com.coravy.maps {
    
    import com.coravy.wiimote.GestureDetection;
    import com.coravy.wiimote.GestureEvent;
    import com.google.maps.controls.ControlBase;
    import com.google.maps.controls.ControlPosition;
    import com.google.maps.controls.PositionControl;
    import com.google.maps.interfaces.IControl;
    import com.google.maps.interfaces.IMap;
    
    import flash.display.DisplayObject;
    import flash.events.*;
    import flash.geom.Point;
    
    import org.wiiflash.Wiimote;
    import org.wiiflash.events.ButtonEvent;
    import org.wiiflash.events.WiimoteEvent;

    /**
     * Diese Klasse zeigt die Standard Kontrollelemente und reagiert auf die Eingabe via Wiimote.
     * Kann der Map als Control hinzugefügt werden.
     * 
     * Es wird ein PositionControl als Delegate benutzt. 
     * Dieser ist für die Anzeige/Kontrolle der Buttons verantwortlich.
     * 
     * @author Stefan Saasen <stefan@coravy.com> 
     */  
    public class WiiControl extends ControlBase {    
                
        private static const STEP:uint = 120;
        
        private var _positionDelegate:IControl;
            
        private var myWiimote:Wiimote = new Wiimote();    
                
        public function WiiControl(pos:ControlPosition = null) {
            if(null == pos) {
                pos = new ControlPosition(ControlPosition.ANCHOR_TOP_LEFT, 10, 10); // DEFAULT
            }
            super(pos);
            
            connectAndInitializeWiimote();
            
            // We use a default PositionControl
            this._positionDelegate = new PositionControl();
        }
                
        override public function getDisplayObject(): DisplayObject {
            return this._positionDelegate.getDisplayObject();
        }
        
        override public function initControlWithMap(m:IMap):void {
            this.map = m;
            this._positionDelegate.initControlWithMap(m);
        }
        
        override public function getSize(): Point {
            return this._positionDelegate.getSize();
        }
        
        override public function setControlPosition(_position:ControlPosition): void {
            this._positionDelegate.setControlPosition(_position);
        }

        private function connectAndInitializeWiimote():void {
            // Connect
            myWiimote.connect();
            
            // Add event listener
			myWiimote.addEventListener(ButtonEvent.UP_RELEASE,    moveUp);
			myWiimote.addEventListener(ButtonEvent.DOWN_RELEASE,  moveDown);
			myWiimote.addEventListener(ButtonEvent.LEFT_RELEASE,  moveLeft);
			myWiimote.addEventListener(ButtonEvent.RIGHT_RELEASE, moveRight);  
			myWiimote.addEventListener(ButtonEvent.PLUS_PRESS,    zoomInAction);  
			myWiimote.addEventListener(ButtonEvent.MINUS_PRESS,   zoomOutAction); 
			// Es sollen nicht nur Buttons verarbeitet werden sondern auch einfache Gesten
			// die auf der Auswertung der Accelerometer Daten beruhen.
			var gs:GestureDetection = new GestureDetection(myWiimote);
			gs.addEventListener(GestureEvent.JERK_DOWN, moveDown);
			gs.addEventListener(GestureEvent.JERK_UP, moveUp);
			gs.addEventListener(GestureEvent.JERK_LEFT, moveLeft);
			gs.addEventListener(GestureEvent.JERK_RIGHT, moveRight); 
        }
        
        private function wiimoteUpdate(e:WiimoteEvent):void {
		    var x:Number  = e.target.sensorX;
            var y:Number  = e.target.sensorY;
		    var z:Number  = e.target.sensorZ;
		    //trace("x: " + x + " y: " + y + " z: " + z);
        }
        
        private function moveUp(e:Event):void {
            moveBy(new Point(0,-STEP));
        }
        
        private function moveDown(e:Event):void {
            moveBy(new Point(0,STEP));
        }

        private function moveLeft(e:Event):void {
            moveBy(new Point(-STEP,0));
        }
        
        private function moveRight(e:Event):void {
            moveBy(new Point(STEP,0));
        }
        
        private function moveBy(p:Point):void {
            this.map.panBy(p);
        }
        
        private function zoomInAction(e:Event) :void {
            trace("zoomInAction");
            this.map.zoomIn();
        }

        private function zoomOutAction(e:Event) :void {
            trace("zoomOutAction");
            this.map.zoomOut();
        }      
    }
 
}